#!/usr/local/bin/python3.sh
# -*- mode: python; indent-tabs-mode: nil; py-indent-offset: 4; coding: utf-8 -*-
# https://stem.torproject.org/tutorials/over_the_river.html

__doc__ = """
Hidden services give you a way of providing a service without exposing your address. These services are only accessible through Tor or Tor2web, and useful for a surprising number of things...

    Hosting an anonymized site. This is usually the first thing that comes to mind, and something we'll demonstrate in a sec.
    Providing an endpoint Tor users can reach without exiting the Tor network. This eliminates the risk of an unreliable or malicious exit getting in the way. Great examples of this are Facebook (facebookcorewwwi.onion) and DuckDuckGo (3g2upl4pq6kufc4m.onion).
    Personal services. For instance you can host your home SSH server as a hidden service to prevent eavesdroppers from knowing where you live while traveling abroad.

Tor2web provides a quick and easy way of seeing if your hidden service is working. To use it simply replace the .onion of your address with .tor2web.org...

This script tests if you can reach a hidden service, passed as an onion address
as an argument. If no argument is given, 3 common onion sites are tested:
Facebook, DuckDuckGo.


"""

import sys
import os
import getpass
import logging

from stem.control import Controller

from stem_examples.tor_controller import get_controller
from stem_examples.stem_utils import vsetup_logging

LOG = logging.getLogger()

lKNOWN_ONIONS = [
  'facebookwkhpilnemxj7asaniu7vnjjbiltxjqhye3mhbshg7kx5tfyd', # facebook
  'duckduckgogg42xjoc72x3sjasowoarfbgcmvfimaftt6twagswzczad', # ddg
]

def iMain(lArgs=None):
    lRetval = lMain(lArgs)
    if lRetval is None:
        return -1
    return 0

def lMain(lArgs=None, timeout=None):
    lRetval = []
    if not lArgs:
        lArgs = lKNOWN_ONIONS
    try:
        password = os.environ.get('TOR_CONTROLLER_PASSWORD')
        if False and os.path.exists('/run/tor/control'):
            controller = get_controller(password=password, unix='/run/tor/control')
        else:
            controller = get_controller(password=password, port=9051)

        for elt in lArgs:
            desc = controller.get_hidden_service_descriptor(elt,
                                                            await_result=True,
                                                            timeout=timeout)
            LOG.info(f"{desc} get_hidden_service_descriptor\n")
            l = desc.introduction_points()
            if not l:
                LOG.warn(f"{elt} NO introduction points {l}\n")
                continue
            LOG.info(f"{elt} introduction points are...\n")

            for introduction_point in l:
                lRetval += [introduction_point]
                LOG.info('  %s:%s => %s' % (introduction_point.address,
                                        introduction_point.port,
                                        introduction_point.identifier))

    except Exception as e:
        LOG.exception(f"Exception: {e}")
    finally:
        del controller
    return lRetval

if __name__ == '__main__':
    if os.environ.get('DEBUG', ''):
        log_level = 10
    else:
        log_level = 20
    vsetup_logging(LOG, log_level)
    try:
        l = lMain(sys.argv[1:])
        if l: print(l)
        i = 0
    except KeyboardInterrupt as e:
        i = 0
    except Exception as e:
        i = 1
    sys.exit(i)
